import sys
import argparse
sys.path.append('/home/jana/MyPython')
import Funkce as fce
import numpy as np
from numpy.random import normal
from scipy.interpolate import splrep, splev
from scipy.optimize import differential_evolution as difevol
from scipy.optimize import minimize
import matplotlib.pyplot as plt
from scipy.stats import norm


class fitLC:
    def __init__(self, hjd, flux, err, synphase, synflux, mc):
	self.hjd = hjd
	self.flux = flux
	self.err = err
	self.per = 7.146644
	self.synphase = synphase
	self.synflux = synflux
	self.mc = mc
	
	if self.mc != False:
	    self.change_noise()
	
    def change_noise(self):
	self.flux = normal(loc=self.flux, scale=self.err)
	

    def get_phase(self, ep):
	phase = fce.quad_phase(self.hjd, ep, self.per)%1
	return phase
    
    def shift_flux(self, flux, dflux):
	flux = flux + dflux
	return flux
      
    def chi2(self, p):
	ep = p[0]
	dy = p[1]

	oph = self.get_phase(ep)
	
	flux = self.shift_flux(self.flux,dy)
	
	tck = splrep(self.synphase, self.synflux)
	sflux = splev(oph, tck)
	
	resid = (flux-sflux)/self.err
	
	return np.sum(resid**2)
      
def corner_plot(block, figname):
  
    #print block
    ncol = len(block[0])
    fig = plt.figure(figsize=(10, 10), dpi=100)
    for i in range(0, len(block[0])):
	for j in range(0, i+1):
	    #print ncol, ncol, ncol*i+j
	    ax = plt.subplot(ncol, ncol, ncol*i+j+1)
	    x = block[:,i]
	    y = block[:,j]
	    if i == j:
		g = norm(loc=x.mean(), scale=x.std(ddof=1))
		n, bins,patches=ax.hist(x,20,normed=True)
		ax.plot(bins, g.pdf(bins), 'r-')
		ax.set_title(r'$p$=%f +/- %f' % (x.mean(), x.std(ddof=1)))
	    else:
		corr = np.mean((x-x.mean())*(y-y.mean()))/(x.std(ddof=1)*y.std(ddof=1))
		ax.hist2d(y,x,20)
		ax.set_title(r'$\rho$=%f' % (corr))
    plt.tight_layout()	    
    plt.savefig(figname)
    #plt.show()      
    plt.close()
		    

def main():
  
    # passes arguments
    prsr = argparse.ArgumentParser()
    prsr.add_argument('-obs', type=str, default=None)
    prsr.add_argument('-syn', type=str, default=None)
    prsr.add_argument('-res', type=str, default=None)
    prsr.add_argument('-err', type=float, default=0.01)
    prsr.add_argument('-mc', action='store_true')
    prsr.add_argument('-niter', type=int, default=1)
    prsr.add_argument('-pmin', action='store_true')
    args = vars(prsr.parse_args())
    #print prsr.parse_args()
    print args
    
    for key in args.keys():
	if key == 'obs':
	    ofile = args[key];
	elif key == 'syn':  
	    sfile = args[key];
	elif key == 'res':  
	    rfile = args[key];  
	elif key == 'err':  
	    err = args[key];
	elif key == 'mc':  
	    mc = args[key];
	elif key == 'niter':  
	    niter = args[key];
	elif key == 'pmin':  
	    pmin = args[key];	
	    
    # parameters
    #ofile = 'U01.smin.01.dat'
    #sfile = 'synU.dat'
    #rfile = 'minima.res'
    #err = 0.017
    #mc = True
    #niter = 1000
    plotIndividual = True
    
    # read the data
    ohjd, oflux = np.loadtxt(ofile, unpack=True, usecols=[0,1])
    sphase, sflux = np.loadtxt(sfile, unpack=True, usecols=[0,1])

    # set initial conditions
    imin = np.argmin(-oflux)
    imax = np.argmax(-oflux)
    if pmin:
	p0 = [ohjd[imin], -oflux[imax]+1.0]   
    else:
	p0 = [ohjd[imin]-7.14664/2., -oflux[imax]+1.0]
    
    resBlock = []
    chi2s=[]
    
    for i in range(0, niter):
        # build the class  
        flc = fitLC(ohjd, oflux, err, sphase, sflux, mc)
        
        #p0 = [ohjd[imin], 0.0]
        #print "Initial: %s chi2: %s" % (str(p0), str(flc.chi2(p0)))
        
        # fit it 
        result = minimize(flc.chi2, p0, method='SLSQP')#, options=dict(xtol=1e-5)) 
        #result = differential_evolution(flc.chi2, [[p]])
        #get output
        p = result.x
        chi2s.append(result.fun)
        resBlock.append(p)
        
        #print "Final: %s chi2:" % str(p), str(result.fun)
        bep = p[0]
        bdflux = p[1]
        
        phase = flc.get_phase(p[0])
        flux = flc.shift_flux(oflux, p[1])
        
        # plots one fit - just to check 
        if plotIndividual == True:
            oph, o = fce.add_wing_phase(phase, flux)
            sph, s = fce.add_wing_phase(sphase, sflux)
            ind = np.argsort(sph)
            
            plt.figure(figsize=(30,10), dpi=100)
            plt.plot(oph, o, 'ro')
            plt.plot(sph, s, 'k-')
            
            if pmin:
		plt.xlim(-0.17, 0.17)
	    else:
		plt.xlim(0.33, 0.67)
	      
            
            plt.savefig(ofile+'.one.png')
            plt.close()
            plotIndividual = False
            plt.close()
            #plt.show()
    
    # filtering of bad results
    chi2s = np.array(chi2s)
    ind = np.where(chi2s<chi2s.mean()+1000*chi2s.std())
    print len(ind)
    plt.plot(chi2s, 'b-')
    plt.savefig(ofile+'.chi2.png')
    plt.close()
    
    resBlock = np.array(resBlock)
    x1 = resBlock[ind,0]
    x2 = resBlock[ind,1]
    
    # plot posterior
    corner_plot(resBlock[ind], ofile+'.png')
    
    # write the uncertaities
    outfile = open(rfile, 'a')
    outfile.write("%20s %15.6f %15.6f %15.6f %15.6f\n" % 
		     (ofile, x1.mean(), x1.std(ddof=1), x2.mean(), x2.std(ddof=1)))
    outfile.close()
    
if __name__ == '__main__':
    main()
      
